<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig\Extension;

use BadFunctionCallException;
use BadMethodCallException;
use Infomaker\Everyware\Support\Str;
use Infomaker\Everyware\Twig\View;
use Twig\TwigFunction;

/**
 * Class Functions
 * @package Infomaker\Everyware\Twig\Extension
 */
class Functions implements ExtensionResourceInterface
{
    /** @var array */
    private $functions = [];

    public function __construct()
    {
        $this->addResource('php_function', [$this, 'phpFunction']);
        $this->addResource('php_method', [$this, 'phpMethod']);
        $this->addResource('render_classes', [$this, 'renderClasses']);
        $this->addResource('render_partial', [$this, 'renderPartial']);
    }

    public function phpFunction($name, $args = [])
    {
        if ( ! function_exists($name)) {
            throw new BadFunctionCallException('Function dose not exist');
        }

        return call_user_func_array($name, $args);
    }

    public function phpMethod($name, $args = [])
    {
        $staticMethod = $name;

        if (is_string($name)) {
            $staticMethod = strpos($name, '::') ? explode('::', $name) : Str::parseCallback($name, '');
        }

        if ( ! method_exists($staticMethod[0], $staticMethod[1])) {
            throw new BadMethodCallException('Method ' . $staticMethod[1] . ' can not be found on ' . $staticMethod[0]);
        }

        return call_user_func_array($staticMethod, $args);
    }

    public function renderClasses($classes): string
    {
        if (func_num_args() > 1) {
            $classes = func_get_args();
        }

        $classes = $this->getValidArray($classes);

        return trim(preg_replace('!\s+!', ' ', implode(' ', $classes)));
    }

    public function renderPartial(string $template, array $data = []): void
    {
        View::render($template, $data);
    }

    /**
     * @inheritDoc
     */
    public function addResource(string $name, $callable = null, array $options = []): void
    {
        $this->functions[$name] = new TwigFunction($name, $callable, $options);
    }

    /**
     * @inheritDoc
     */
    public function getResourceList(): array
    {
        return $this->functions;
    }

    private function getValidArray($value)
    {
        if ( ! \is_iterable($value)) {
            $value = (array)$value;
        }

        if ($value instanceof \Traversable) {
            return iterator_to_array($value);
        }

        return $value;
    }
}
